# mcs_tzi1.py
# A. Srinivasan and S. Ellingson, Aug 7, 2009
# "Tier Zero" interface code for direct communication with subsystems
# Usage:
#   $ python mcs_tzi1.py <DEST_NAME> <DEST_IP> <TX_PORT> <RX_PORT> <CMD> <REF> <DATA>
#or $ python mcs_tzi1.py <DEST_NAME> <DEST_IP> <TX_PORT> <RX_PORT> <CMD> <REF> -f <DATA_FILE>  
#   <DEST_NAME>; e.g. SHL. (source name is automatically MCS)
#   <DEST_IP>; e.g., 127.0.0.1 for local loopback
#   <TX_PORT>; e.g., 1738
#   <RX_PORT>; e.g., 1739
#   <CMD>; e.g., PNG.  No error checking is done.
#   <REF>; e.g., 1361
#   <DATA> is whatever should go in the DATA field.  Enter as a string with single quotes.
#   <DATA_FILE> is the file that contains all the data that needs to go in the DATA field, in any format
# Examples:
# $ python mcs_tzi1.py SHL 127.0.0.1 1738 1739 RPT 1361 'VERSION'
# $ python mcs_tzi1.py SHL 127.0.0.1 1738 1739 RPT 1361 -f data_field_contents.dat
 
import socket
import time
import datetime
import math
import string
import struct   # for packing of binary to/from strings
import sys

from optparse import OptionParser
parser = OptionParser()
parser.add_option("-f")
(options,args) = parser.parse_args()

if (len(sys.argv)<7):
    print 'More arguments expected. No action taken'
    exit()
DEST_NAME = sys.argv[1]
DEST_IP = sys.argv[2]
PORTT = int(sys.argv[3])# TX_PORT
PORTR = int(sys.argv[4]) # RX_PORT
cmd = sys.argv[5]
ref = sys.argv[6] # reference
if (len(sys.argv)>7):
    if (sys.argv[7] == '-f'):
        data_file = sys.argv[8]
        file_pointer = open(data_file,'rb')
        data = file_pointer.read()
        data_print = '' # data from a file not displayed
         
    else:    
        data = sys.argv[7]
        data_print = data
else:
    data = ''
    data_print = data

B = 8192 # Buffer Size


#==============================
# Message Processing Subroutine
#==============================

def command(iref,cmd,data,data_print):
    # (in)  iref:    [int] message REFERENCE number to use 
    # (in)  cmd:     [str] three-letter command; e.g., "PNG"
    # (in)  data:    [str] DATA field of message
    # (out) err:     [int] Error code: sum of the following:
    #                   0: no errors
    #                   1: (not currently used)
    #                   2: wrong "DESTINATION"
    #                   4: R-RESPONSE = "R" (command reject indicator) received
    # (out) summary: [str] R-SUMMARY (i.e., sender's MIB 1.1)
    # (out) rdata:   [str] Contents of DATA field with R-* fields stripped away

    # ---------------------
    # Set up sockets
    # ---------------------

    # Set up the receive socket for UDP
    r = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    r.bind(('', PORTR))  # Accept connections from anywhere
    r.setblocking(1)    # Blocking on this socket 
    r.settimeout(3)     # Set timeout to 3 s

    # Set up transmit socket for UDP
    t = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    t.connect((DEST_IP, PORTT))

    # -------------------
    # Message Preparation
    # -------------------

    # determine current time
    dt = datetime.datetime.utcnow()
    year        = dt.year
    month       = dt.month
    day         = dt.day
    hour        = dt.hour
    minute      = dt.minute
    second      = dt.second
    millisecond = dt.microsecond / 1000

    # construct the MJD field
    # adapted from http://paste.lisp.org/display/73536
    # can check result using http://www.csgnetwork.com/julianmodifdateconv.html
    a = (14 - month) // 12
    y = year + 4800 - a
    m = month + (12 * a) - 3
    p = day + (((153 * m) + 2) // 5) + (365 * y)
    q = (y // 4) - (y // 100) + (y // 400) - 32045
    mjdi = int(math.floor( (p+q) - 2400000.5))
    mjd = string.rjust(str(mjdi),6)
    #print '#'+mjd+'#'

    # construct the MPM field
    mpmi = int(math.floor( (hour*3600 + minute*60 + second)*1000 + millisecond ) )
    mpm = string.rjust(str(mpmi),9) 
    #print '#'+mpm+'#'

    # Build the message
    # Note we are just using a single, non-updating REFERENCE number in this case
    message =           DEST_NAME
    message = message + 'MCS'
    message = message + cmd
    message = message + string.rjust(str(iref),9)
    message = message + string.rjust(str(len(data)),4)
    message = message + str(mjd)
    message = message + str(mpm) + ' '  # note trailing space
    message_print = message
    message = message + data
    for j in range(len(data_print)):
        if (ord(data_print[j])<32 and  ord(data_print[j])>126): #ascii values of non-printable characters
            data_print[j] = ' ' #removing non-printable characters 
    if (len(data_print)>45): #truncating data displayed to 45 bytes
       data_print = data_print[:45]

    message_print = message_print + data_print
    
    # ---------------------
    # Sending and Receiving
    # ---------------------

    # Send it and wait for response
    t.send(message)            # send the message
    print message_print # say what was sent
    #response = ''
    response = r.recv(B)       # this blocks until the server responds
    response_data = response[38:]
    for j in range(len(response_data)):
        if (ord(response_data[j])<32 and  ord(response_data[j])>126): #ascii values of non-printable characters
            response_data[j] = ' ' #removing non-printable characters 
    if (len(response_data)>45):
       response_data = response_data[:45]
    response_print = response[:38]+response_data
    print response_print
    r.close() # close receive socket
    t.close() # close transmit socket


# attempting to connect to the subsystem
command(ref,cmd,data,data_print)

